#!/usr/bin/env python3

#
# SPDX-License-Identifier: BSD-3-Clause
# Copyright Contributors to the OpenEXR Project.
#

from __future__ import print_function
import sys
import os
import tempfile
import atexit
import unittest
import numpy as np

import OpenEXR

class TestRGBA(unittest.TestCase):

    def do_rgb(self, array_dtype):

        # Construct an RGB channel

        height = 5
        width = 4
        nrgba = 3
        size = width * height * nrgba
        RGB = np.array([i for i in range(0,size)], dtype=array_dtype).reshape((height, width, nrgba))
        channels = { "RGB" : OpenEXR.Channel(RGB) }

        header = {}
        outfile = OpenEXR.File(header, channels)

        outfile.write("out.exr")

        # 
        # Read as separate channels
        #

        infile = OpenEXR.File("out.exr", separate_channels=True)

        R = infile.channels()["R"].pixels
        G = infile.channels()["G"].pixels
        B = infile.channels()["B"].pixels

        shape = R.shape
        width = shape[1]
        height = shape[0]
        for y in range(0,height):
            for x in range(0,width):
                r = R[y][x]
                g = G[y][x]
                b = B[y][x]
                self.assertEqual(r, RGB[y][x][0])
                self.assertEqual(g, RGB[y][x][1])
                self.assertEqual(b, RGB[y][x][2])

        #
        # Read as RGB channel
        #

        infile = OpenEXR.File("out.exr")

        inRGB = infile.channels()["RGB"].pixels
        shape = inRGB.shape
        width = shape[1]
        height = shape[0]
        self.assertEqual(shape[2], 3)

        self.assertTrue(np.array_equal(inRGB, RGB))
        os.remove("out.exr")

    def do_rgba(self, array_dtype):

        # Construct an RGB channel

        height = 6
        width = 5
        nrgba = 4
        size = width * height * nrgba
        RGBA = np.array([i for i in range(0,size)], dtype=array_dtype).reshape((height, width, nrgba))
        channels = { "RGBA" : OpenEXR.Channel(RGBA) }

        header = {}
        outfile = OpenEXR.File(header, channels)

        outfile.write("out.exr")

        # 
        # Read as separate channels
        #

        infile = OpenEXR.File("out.exr", separate_channels=True)

        R = infile.channels()["R"].pixels
        G = infile.channels()["G"].pixels
        B = infile.channels()["B"].pixels
        A = infile.channels()["A"].pixels

        shape = R.shape
        width = shape[1]
        height = shape[0]
        for y in range(0,height):
            for x in range(0,width):
                r = R[y][x]
                g = G[y][x]
                b = B[y][x]
                a = A[y][x]
                self.assertEqual(r, RGBA[y][x][0])
                self.assertEqual(g, RGBA[y][x][1])
                self.assertEqual(b, RGBA[y][x][2])
                self.assertEqual(a, RGBA[y][x][3])

        #
        # Read as RGBA channel
        #

        infile = OpenEXR.File("out.exr")

        inRGBA = infile.channels()["RGBA"].pixels
        shape = inRGBA.shape
        width = shape[1]
        height = shape[0]
        self.assertEqual(shape[2], 4)

        self.assertTrue(np.array_equal(inRGBA, RGBA))
        os.remove("out.exr")

    def do_rgba_prefix(self, array_dtype):

        # Construct an RGB channel

        height = 6
        width = 5
        nrgba = 4
        size = width * height
        RGBA = np.array([i for i in range(0,size*nrgba)], dtype=array_dtype).reshape((height, width, nrgba))
        Z = np.array([i for i in range(0,size)], dtype=array_dtype).reshape((height, width))
        channels = { "left" : OpenEXR.Channel(RGBA), "left.Z" : OpenEXR.Channel(Z) }

        header = {}
        outfile = OpenEXR.File(header, channels)

        print(f"write out.exr")
        outfile.write("out.exr")

        # 
        # Read as separate channels
        #

        print(f"read out.exr as single channels")
        infile = OpenEXR.File("out.exr", separate_channels=True)

        R = infile.channels()["left.R"].pixels
        G = infile.channels()["left.G"].pixels
        B = infile.channels()["left.B"].pixels
        A = infile.channels()["left.A"].pixels
        Z = infile.channels()["left.Z"].pixels

        shape = R.shape
        width = shape[1]
        height = shape[0]
        for y in range(0,height):
            for x in range(0,width):
                r = R[y][x]
                g = G[y][x]
                b = B[y][x]
                a = A[y][x]
                self.assertEqual(r, RGBA[y][x][0])
                self.assertEqual(g, RGBA[y][x][1])
                self.assertEqual(b, RGBA[y][x][2])
                self.assertEqual(a, RGBA[y][x][3])

        #
        # Read as RGBA channel
        #

        print(f"read out.exr as rgba channels")
        infile = OpenEXR.File("out.exr")

        inRGBA = infile.channels()["left"].pixels
        shape = inRGBA.shape
        width = shape[1]
        height = shape[0]
        self.assertEqual(shape[2], 4)
        inZ = infile.channels()["left.Z"].pixels

        self.assertTrue(np.array_equal(inRGBA, RGBA))
        os.remove("out.exr")

    def test_rgb_uint32(self):
        self.do_rgb('uint32')

    def test_rgb_f(self):
        self.do_rgb('f')

    def test_rgba_uint32(self):
        self.do_rgba('uint32')

    def test_rgba_prefix_uint32(self):
        self.do_rgba_prefix('uint32')

    def test_rgba_f(self):
        self.do_rgba('f')

if __name__ == '__main__':
    unittest.main()
    print("OK")





#!/usr/bin/env python

# SPDX-License-Identifier: BSD-3-Clause
# Copyright (c) Contributors to the OpenEXR Project.

import sys, os
from do_run import do_run

print(f"testing exrheader: {sys.argv}")

exrheader = sys.argv[1]
image_dir = sys.argv[2]
version = sys.argv[3]

test_images = {}
test_images["GrayRampsHorizontal"] = f"{image_dir}/GrayRampsHorizontal.exr"

# no args = usage message
result = do_run ([exrheader], True)
assert result.stderr.startswith ("Usage: ")

# -h = usage message
result = do_run ([exrheader, "-h"])
assert result.stdout.startswith ("Usage: ")

# --help = usage message
result = do_run ([exrheader, "--help"])
assert result.stdout.startswith ("Usage: ")

# --version
result = do_run ([exrheader, "--version"])
assert result.stdout.startswith ("exrheader")
assert version in result.stdout

# nonexistent.exr, error
result = do_run ([exrheader, "nonexistent.exr"], True)

def find_line(keyword, lines):
    for line in lines:
        if line.startswith(keyword):
            return line
    return None

# attributes
result = do_run ([exrheader, test_images["GrayRampsHorizontal"]])

output = result.stdout.split('\n')
try:
    assert "2, flags 0x0" in find_line("file format version:", output)
    assert "pxr24" in find_line ("compression", output)
    assert "(0 0) - (799 799)" in find_line ("dataWindow", output)
    assert "(0 0) - (799 799)" in find_line ("displayWindow", output)
    assert "increasing y" in find_line ("lineOrder", output)
    assert "1" in find_line ("pixelAspectRatio", output)
    assert "(0 0)" in find_line ("screenWindowCenter", output)
    assert "1" in find_line ("screenWindowWidth", output)
    assert "scanlineimage" in find_line ("type (type string)", output)
except AssertionError:
    print(result.stdout)
    raise
print("success")









